import { Awaitable } from '@antfu/utils';
import { SnapshotStateType, SnapshotResolver } from 'jest-snapshot';
import { SnapshotStateOptions } from 'jest-snapshot/build/State';

interface UncheckedSnapshot {
    filePath: string;
    keys: Array<string>;
}
interface SnapshotSummary {
    added: number;
    didUpdate: boolean;
    failure: boolean;
    filesAdded: number;
    filesRemoved: number;
    filesRemovedList: Array<string>;
    filesUnmatched: number;
    filesUpdated: number;
    matched: number;
    total: number;
    unchecked: number;
    uncheckedKeysByFile: Array<UncheckedSnapshot>;
    unmatched: number;
    updated: number;
}

interface Context {
    file: string;
    title?: string;
    fullTitle?: string;
}
declare class SnapshotManager {
    snapshotState: SnapshotStateType | null;
    snapshotSummary: SnapshotSummary;
    snapshotOptions: SnapshotStateOptions;
    context: Context | null;
    testFile: string;
    snapshotResolver: SnapshotResolver | null;
    rootDir: string;
    constructor({ rootDir, update, snapshotResolver, }: {
        rootDir: string;
        update?: boolean;
        snapshotResolver?: SnapshotResolver | null;
    });
    onFileChanged(): void;
    setTask(task: Task): void;
    setContext(context: Context): void;
    assert(received: unknown, message: string): void;
    clear(): void;
    saveSnap(): void;
    report(): string[] | undefined;
}

interface UserOptions {
    /**
     * Include globs for test files
     *
     * @default ['**\/*.test.ts']
     */
    includes?: string[];
    /**
     * Exclude globs for test files
     * @default ['**\/node_modules\/**']
     */
    excludes?: string[];
    /**
     * Handling for dependencies inlining or externalizing
     */
    deps?: {
        external?: (string | RegExp)[];
        inline?: (string | RegExp)[];
    };
    /**
     * Register apis globally
     *
     * @default false
     */
    global?: boolean;
    /**
     * Use `jsdom` or `happy-dom` to mock browser APIs
     *
     * @default false
     */
    dom?: boolean | 'jsdom' | 'happy-dom';
    /**
     * Run tests files in parallel
     *
     * @default false
     */
    parallel?: boolean;
    /**
     * Update snapshot files
     *
     * @default false
     */
    update?: boolean;
    /**
     * Watch mode
     *
     * @default false
     */
    watch?: boolean;
    /**
     * Project root
     */
    root?: string;
    /**
     * Custom reporter for output
     */
    reporter?: Reporter;
}
interface ResolvedConfig extends Required<UserOptions> {
    filters?: string[];
    config?: string;
}
declare type RunMode = 'run' | 'skip' | 'only' | 'todo';
declare type TaskState = RunMode | 'pass' | 'fail';
interface ConcurrentOptions {
    timeout: number;
}
interface Task {
    name: string;
    mode: RunMode;
    concurrent?: ConcurrentOptions;
    suite: Suite;
    fn: () => Awaitable<void>;
    file?: File;
    state?: TaskState;
    error?: unknown;
}
declare type TestFunction = () => Awaitable<void>;
interface ConcurrentCollector {
    (name: string, fn: TestFunction, timeout?: number): void;
    only: (name: string, fn: TestFunction, timeout?: number) => void;
    skip: (name: string, fn: TestFunction, timeout?: number) => void;
    todo: (name: string) => void;
}
interface OnlyCollector {
    (name: string, fn: TestFunction): void;
    concurrent: (name: string, fn: TestFunction, timeout?: number) => void;
}
interface SkipCollector {
    (name: string, fn: TestFunction): void;
    concurrent: (name: string, fn: TestFunction, timeout?: number) => void;
}
interface TodoCollector {
    (name: string): void;
    concurrent: (name: string) => void;
}
interface TestCollector {
    (name: string, fn: TestFunction): void;
    concurrent: ConcurrentCollector;
    only: OnlyCollector;
    skip: SkipCollector;
    todo: TodoCollector;
}
declare type HookListener<T extends any[]> = (...args: T) => Awaitable<void>;
interface Suite {
    name: string;
    mode: RunMode;
    tasks: Task[];
    file?: File;
    error?: unknown;
    status?: TaskState;
    hooks: {
        beforeAll: HookListener<[Suite]>[];
        afterAll: HookListener<[Suite]>[];
        beforeEach: HookListener<[Task, Suite]>[];
        afterEach: HookListener<[Task, Suite]>[];
    };
}
interface SuiteCollector {
    readonly name: string;
    readonly mode: RunMode;
    test: TestCollector;
    collect: (file?: File) => Promise<Suite>;
    clear: () => void;
    on: <T extends keyof Suite['hooks']>(name: T, ...fn: Suite['hooks'][T]) => void;
}
declare type TestFactory = (test: (name: string, fn: TestFunction) => void) => Awaitable<void>;
interface File {
    filepath: string;
    suites: Suite[];
    collected: boolean;
    error?: unknown;
}
interface RunnerContext {
    filesMap: Record<string, File>;
    files: File[];
    suites: Suite[];
    tasks: Task[];
    config: ResolvedConfig;
    reporter: Reporter;
    snapshotManager: SnapshotManager;
}
interface GlobalContext {
    suites: SuiteCollector[];
    currentSuite: SuiteCollector | null;
}
interface Reporter {
    onStart?: (config: ResolvedConfig) => Awaitable<void>;
    onCollected?: (files: File[], ctx: RunnerContext) => Awaitable<void>;
    onFinished?: (ctx: RunnerContext, files?: File[]) => Awaitable<void>;
    onSuiteBegin?: (suite: Suite, ctx: RunnerContext) => Awaitable<void>;
    onSuiteEnd?: (suite: Suite, ctx: RunnerContext) => Awaitable<void>;
    onFileBegin?: (file: File, ctx: RunnerContext) => Awaitable<void>;
    onFileEnd?: (file: File, ctx: RunnerContext) => Awaitable<void>;
    onTaskBegin?: (task: Task, ctx: RunnerContext) => Awaitable<void>;
    onTaskEnd?: (task: Task, ctx: RunnerContext) => Awaitable<void>;
    onWatcherStart?: (ctx: RunnerContext) => Awaitable<void>;
    onWatcherRerun?: (files: string[], trigger: string, ctx: RunnerContext) => Awaitable<void>;
}

export { ConcurrentOptions as C, File as F, GlobalContext as G, HookListener as H, ResolvedConfig as R, SuiteCollector as S, TestFunction as T, UserOptions as U, TestFactory as a, Suite as b, RunMode as c, TaskState as d, Task as e, TestCollector as f, RunnerContext as g, Reporter as h };
